/******************************************************************************
Copyright 2004-16 VERIFONE DANMARK A/S    All rights reserved.

File Name:  flex.h
Contents:   Include file

******************************************************************************/

#ifndef __FLEX_H
#define __FLEX_H


#ifdef __cplusplus
extern "C" {  // only need to export C interface if
#endif        // used by C++ source code

#ifdef _WIN32
#define DllExport __declspec(dllexport)
#else
#define DllExport
#endif

#define MAX_RECEIPT_SIZE			16384

#ifndef TRUE
#define TRUE 1
#define true TRUE
#endif
#ifndef FALSE
#define FALSE 0
#define false FALSE
#endif

// Generel return value
// CONNECT RETURN VALUES	flxConnect, flxOpen, return values
#define SUCCESS						0x00
#define FAILURE						0x01

//flxOpen return value - depricated
#define OPEN_NO_RECEIPT				0x13
#define CONNECT_OK					0x00
#define CONNECT_COMPORTINITFAILURE	0x03
#define CONNECT_SW_NOTCOMPATIBLE	0x04
#define CONNECT_TERM_NOT_RESP		0x05
#define CONNECT_COMPORTOPEN			0x06

// GENERAL flx RETURN VALUES
#define DATALINKERROR				0x07
#define FUNCTION_NOT_POSSIBLE		0x08
#define TIMEOUT_IN_COMMUNICATION	0x09
#define CONNECT_NO_LICENSE          0x0A
#define CONNECT_INTERNAL_ERROR      0x0B
#define CONNECT_SYSTEM_ERROR		0x0C
#define CONNECT_KISS_ERROR			0x0D
#define	CONNECT_TLV_NOT_ALLOWED		0x0E

// Status return codes from flxIdle
#define flxIdle(x,y)	flxIdleIPforwarding(x,y)


enum _SYNC
{
	CARD_SWIPED	=				0x20,
	CARD_REMOVED =				0x21,
	TEDO =						0x5465446f			// teleload done
};

// flxCardTransaction return values when local card
#define LOCALCARD_OK                0x10
#define LOCALCARD_NOT_OK            0x11

// Trace level defines, tracefile placed in cwd
enum CONF_E 
{
	CONF_PARAM = 1,		// Placement of inifile
	CONF_TRACE,			// Enable trace
	CONF_EXTTRACE,		// Enable extensive tracing
	CONF_EXTTRACE_PLUS,	// Enable extensive tracing with append
	CONF_FILEPATH,		// Define directory path for flxdrv.ini and trace files
	CONF_KISSPARAM,
	CONF_NO_TRACE_AT_ALL=16,
	CONF_RESETPORT,
	CONF_GUARDTIME,
	CONF_RECEIPT_WIDTH,
	CONF_PIP_TIMEOUT,
	CONF_DOWNLOAD_TIMEOUT,
	CONF_DOWNLOAD_BLOCK_SIZE,
	CONF_DOWNLOAD_TIMEOUT_NEXT,
	CONF_DOWNLOAD_BLOCK_SIZE_NEXT,
	CONF_DOWNLOAD_METODE,
	CONF_IP_ROUTING_RECV_BUFFER_SIZE,
	CONF_USE_GETTIMEOFDAY,
};

// Terminal/ECR communication communication type
enum COMM_E {
	RS232_COMM = 1,
	ETHERNET_COMM,
	USB_COMM
};

// CALLBACK METHODS
typedef enum FLX_CALLBACK_E 
{
	FLX_CALLBACK_SET_CARDDATA = 0,
	FLX_CALLBACK_PRINT_RECEIPT,
	FLX_CALLBACK_DISPLAY_STATUS,
	FLX_CALLBACK_ABORT,
	FLX_CALLBACK_EXTCARDCONFIRM,
	FLX_CALLBACK_VERSIGCONFIRM,				// 5
	FLX_CALLBACK_ADVICEFLAG,
	FLX_CALLBACK_SET_AMOUNT,				// send amount to terminal from ecr
	FLX_CALLBACK_SET_AMOUNT_FEE,			// send fee to terminal from ecr
	FLX_CALLBACK_GET_AMOUNT_FEE,			// receive fee from terminal to ecr
	FLX_CALLBACK_STATUSTRACE,				// 10
	FLX_CALLBACK_STATETRACE,
	FLX_CALLBACK_ADVICELOG,
	FLX_CALLBACK_MENU,
	FLX_CALLBACK_MENURESULT,
	FLX_CALLBACK_GET_BINARY_RECEIPT,		// 15
	FLX_CALLBACK_EARLY_STAN_PAN,
	FLX_CALLBACK_BREAK_IP,
	FLX_CALLBACK_TIMER,
	FLX_CALLBACK_PUT_TOKEN,
	FLX_CALLBACK_GET_TOKEN,					// 20
	FLX_CALLBACK_SET_AMOUNT_GRATUITY,		// send gratuity to terminal from ecr
	FLX_CALLBACK_STOPLIST,
	FLX_CALLBACK_PRE_RESULT,
	FLX_CALLBACK_SET_ECR_EXTENDED_FUNCTIONS,
	FLX_CALLBACK_HOSTADVICE,				// 25
	FLX_CALLBACK_SPARE01,					// was carddata
	FLX_CALLBACK_SET_EXTENDED_AMOUNT,
	FLX_CALLBACK_GET_EXTENDED_AMOUNT,
	FLX_CALLBACK_EIE_DATA_RECEIVED,
	FLX_CALLBACK_EIE_DATA2HOST,				// 30
	FLX_CALLBACK_PUT_SCAN_BAR_CODE,
	FLX_CALLBACK_EXTRA_APP_DATA_RECEIVED,
	FLX_CALLBACK_CARD_SWIPE
} FLX_CALLBACK;

enum FLX_TPROPS_INDEX_E
{
	FLX_TPROPS_INDEX_TERMINALID,		// 00
	FLX_TPROPS_INDEX_RECEIPTTYPE,
	FLX_TPROPS_INDEX_GRATUITYMODEL,
	FLX_TPROPS_INDEX_DCC,
	FLX_TPROPS_INDEX_TOKEN,
	FLX_TPROPS_INDEX_PREPAID,			// 05
	FLX_TPROPS_INDEX_KEYENTRY,
	FLX_TPROPS_INDEX_OFFLINE,
	FLX_TPROPS_INDEX_IP_ROUTING,
	FLX_TPROPS_INDEX_MERCHANTNUMBER,
	FLX_TPROPS_INDEX_MERCHANTNAME,		// 10
	FLX_TPROPS_INDEX_MERCHANTCITY,
	FLX_TPROPS_INDEX_MERCHANTADDRESS,
	FLX_TPROPS_INDEX_MERCHANTZIP,
	FLX_TPROPS_INDEX_MERCHANTPHONE,
	FLX_TPROPS_INDEX_MERCHANTBRN,		// 15
	FLX_TPROPS_INDEX_FLXTRACELEVEL,		// 16
	FLX_TPROPS_INDEX_LANGUAGE,
	FLX_TPROPS_INDEX_VAT,
	FLX_TPROPS_INDEX_USERINPUT,
	FLX_TPROPS_INDEX_COUNTRYCODE,		// 20
	FLX_TPROPS_INDEX_DEFAULTCURRENCY,
	FLX_TPROPS_INDEX_SPARE,
	FLX_TPROPS_INDEX_TERMINALTYPE,
	FLX_TPROPS_INDEX_DCCMARKUP,
	FLX_TPROPS_INDEX_TCS,				// 25
	FLX_TPROPS_INDEX_SOFTWARE_VERSION,
	FLX_TPROPS_INDEX_CDP,
	FLX_TPROPS_INDEX_PSAM_CDP,
	FLX_TPROPS_INDEX_HARDWARE_NAME,
	FLX_TPROPS_INDEX_EIE,				// 30
	FLX_TPROPS_INDEX_PSAM_EIE,			// 31
	FLX_TPROPS_INDEX_ISSUER_ENVELOPE_NON_EMV_SIZE,
	FLX_TPROPS_INDEX_ISSUER_ENVELOPE_EMV_SIZE,
	FLX_TPROPS_INDEX_TOTAL_ISSUER_ENVELOPE_NON_EMV_SIZE,
	FLX_TPROPS_INDEX_TOTAL_ISSUER_ENVELOPE_EMV_SIZE,
	FLX_TPROPS_INDEX_ERECEIPT_SCHEMES,
	FLX_TPROPS_INDEX_PSAM_VERSION,
	FLX_TPROPS_INDEX_CONTACTLESSFLAGS,
	FLX_TPROPS_INDEX_BUILD_DATE,
	FLX_TPROPS_INDEX_MAX				// and keep lsat
};

// Abort callback return values
#define OPERATORWISHTOABORT			0x01
#define OPERATORDONOTWISHTOABORT	0x02

// Receipt callback return values
#define RECEIPT_OK					0x01
#define RECEIPT_NOTOK				0x00

// verSigConfirmation
// used in resultMenu (and stopList)
#define SIGNATURE_OK				0x01
#define SIGNATURE_NOTOK				0x00
#define SIGNATURE_NOTKNOWNYET		0x02

// Carddata callback return values
#define CARDNUMBER_OK				0x01	// Card OK, (external card OK do not confirm amount)
#define CARDNUMBER_OK_CONFIRM       0x03	// External card OK and confirm amount.
#define CARDNUMBER_NOTOK			0x00
#define CARDNUMBER_OK_ECR_ACTION0	0x04
#define CARDNUMBER_OK_ECR_ACTION1	0x05
#define CARDNUMBER_OK_ECR_ACTION2	0x06
#define CARDNUMBER_OK_ECR_ACTION3	0x07
#define CARDNUMBER_OK_ECR_ACTION4	0x08
#define CARDNUMBER_OK_ECR_ACTION5	0x09
#define CARDNUMBER_OK_ECR_ACTION6	0x0A
#define CARDNUMBER_OK_ECR_ACTION7	0x0B
#define CARDNUMBER_OK_ECR_ACTION8	0x0C
#define CARDNUMBER_OK_ECR_ACTION9	0x0D
#define CARDNUMBER_OK_ECR_ACTION10	0x0E
#define CARDNUMBER_OK_ECR_ACTION11	0x0F
#define CARDNUMBER_OK_ECR_ACTION12	0x10
#define CARDNUMBER_OK_ECR_ACTION13	0x11
#define CARDNUMBER_OK_ECR_ACTION14	0x12
#define CARDNUMBER_OK_ECR_ACTION15	0x13
#define CARDNUMBER_OK_ECR_ACTION16	0x14

#define CARDNUMBER_OK_SILENTABORT	0x80


// cardType values
#define PBS_CARD					0x00
#define OTHER_CARD					0x01


// TRANSACTION METHODS
// NOTE: FLX_TRANS_XXX_PURCHASE and FLX_TRANS_XXX_REFUND extendable by MI.
//       These definitions are private for flexdriver and NOT equal to OTRS TT
typedef enum KEY_ENTRY_E
{
	KEY_ENTRY = 0x80,
} KEY_ENTRYTYPE;

typedef enum FLX_TRANSTYPE_E 
{
	FLX_TRANS_PURCHASE = 0,					// Debit transaction
	FLX_TRANS_REFUND,						// Credit transaction
    FLX_TRANS_ORIGINAL_AUTH,				// Original Authorization, outputs a token
    FLX_TRANS_SUPP_AUTH,					// Token based transaction
    FLX_TRANS_CAPTURE,						// Token based transaction
    FLX_TRANS_REVERSAL_AUTH,				// Token based transaction	
	FLX_TRANS_PREPAID_PURCHASE,				// Debit prepaid transaction
	FLX_TRANS_PREPAID_REFUND,				// Credit prepaid transaction
	FLX_TRANS_PREPAID_ORIGINAL_AUTH,		// Orig auth prepaid transaction, no token output
	FLX_TRANS_PREPAID_SCAN_PURCHASE,		// Debit prepaid scan transaction
	FLX_TRANS_PREPAID_SCAN_REFUND,			// Credit prepaid scan transaction
	FLX_TRANS_PREPAID_SCAN_ORIGINAL_AUTH,   // Orig auth prepaid scan transaction, no token output
	FLX_TRANS_CANCELLATION,
	FLX_TRANS_CARDCHECK,					// card check
	FLX_TRANS_POST_PURCHASE,
	FLX_TRANS_POST_REFUND
} FLX_TRANSTYPE;


// ERROR CODES
typedef enum ERR_E 
{
	ERR_UNKNOWN = 0x00010000,
	ERR_TERM_NOT_READY,
	ERR_NO_CONNECTION,
	ERR_NO_RECEIPT_NOT_USED,
	ERR_SW_NOTCOMPATIBLE,
	ERR_NO_LICENSE,
	ERR_TOKEN_TRANS,
	ERR_SYSTEM_ERROR,
	ERR_SPARE,
	ERR_STATE,
	ERR_TOKEN,
	ERR_IP_ROUTING,
	ERR_TRANSMISSION,
	ERR_STOP_TIMEOUT,
	ERR_TLV_NOT_ALLOWED,
} ERRORCODE_TYPE;

// ADMINISTRATION FUNCTIONS
typedef enum ADMIN_E 
{
    ADMIN_ENDOFDAY = 1,
    ADMIN_ENDOFDAYLOG,
    ADMIN_REPORT_TERMINALREPORT,
    ADMIN_REPORT_TOTALS,
    ADMIN_REPORT_LOG,				// 5
    ADMIN_REPORT_OLDLOG,
    ADMIN_LASTRECEIPT,
    ADMIN_UNLOCK_RECEIPT,
    ADMIN_CLOCKSYNCPBS,
    ADMIN_CLOCKSYNCPOINT,			//10
    ADMIN_SENDLOG,
    ADMIN_CLEARDATASTORE,
    ADMIN_DOWNLOADPROGRAM,
    ADMIN_DOWNLOADPARAM,
    ADMIN_DOWNLOADPAN,				//15
    ADMIN_DOWNLOADTLCMDB,
    ADMIN_RESTORETLCMDB,
    ADMIN_CONTRASTUP,
    ADMIN_CONTRASTDOWN,
    ADMIN_RESTARTTERMINAL,			//20
    ADMIN_EJECTCARD,
    ADMIN_MSC,						// For future use (fetch the card range table)
    ADMIN_BACKLIGHT_ON,
    ADMIN_BACKLIGHT_OFF,
	ADMIN_NETWORK_REPORT,			// 25
	ADMIN_RATES_REPORT,
	ADMIN_GRATUITY_RECEIPT,
	ADMIN_EXCLUDE_DATASTORE_RECORD_WITH_STAN,		// for internal use - uses flxTerminalProperties
    ADMIN_ADVICEFORWARDING,
	ADMIN_REPORT_FILE5STATUS,
    ADMIN_RESERVED_FOR_OCX = 31,    
    ADMIN_REPORT_PCT,
	ADMIN_DOWNLOADDCCRATES,
	ADMIN_UPDATEPSAM,
	ADMIN_UPDATEFEETABLE,			// 35
    ADMIN_UPDATESALT,
	ADMIN_GETADVICERECON,							// for internal use - uses flxTerminalProperties
	ADMIN_SET_BATCH_NUMBERS,						// for internal use - uses flxTerminalProperties
    ADMIN_REPORT_TCS,
	ADMIN_REPORT_TPROPS_CVS,		// 40
	ADMIN_EVENTLOG_PRINT,
	ADMIN_EVENTLOG_SEND,							// for internal use - uses flxTerminalProperties
	ADMIN_EVENTLOG_DELETE,							// for internal use - uses flxTerminalProperties
	ADMIN_GETIP_SETTINGS,							
    ADMIN_SETIP_SETTINGS,			// 45			// for internal use - uses flxTerminalProperties
	ADMIN_DOWNLOAD_IMAGES,			// 
	ADMIN_CHECK_CARD,
	ADMIN_GETTELEDONE_SETTINGS,							
    ADMIN_SETTELEDONE_SETTINGS,		    			// for internal use - uses flxTerminalProperties
	ADMIN_REPORT_CTLSREPORT_SHORT,
    ADMIN_USER_INPUT,		    					// for internal use - uses flxTerminalProperties
	ADMIN_SWIPP_ID,
	ADMIN_REPORT_CTLSREPORT_LONG,
	ADMIN_MAX,						// last entry
} FLX_ADMIN_FUNCTION;

// ADDITIONAL COMMANDS (not for use in flxAdministration)
#define ADMIN_GET_DC_PROPERTIES_STAN	0x80

#define ADMIN_GET_ID					0x83
#define ADMIN_SET_ECR_EXTENDED_FUNCTIONS	0x84
#define ADMIN_GET_ECR_EXTENDED_FUNCTIONS	0x85

typedef enum FLX_AMOUNT_T
{
	FLX_AMOUNT_AMOUNT,
	FLX_AMOUNT_FEE,
	FLX_AMOUNT_GRATUITY,
	FLX_AMOUNT_VAT,
	FLX_AMOUNT_BACK
} FLX_AMOUNT_TYPE;

// CURRENCY CODES in currency.h
typedef enum CURR_CODE_E 
{
	_DKK = 208,		// DK Kroner
	_ISK = 352,		// IS Kroner
	_JPY = 392,		// JP Yen
	_NOK = 578,		// N  Kroner
	_SEK = 752,		// S  Kroner
	_CHF = 756,		// SW Francs
	_GBP = 826,		// GB Pound
	_USD = 840,		// US Dollar
	_EUR = 978,		//    Euro
} CURR_CODES;

// Merchant Initiative
typedef enum FLX_MI_E 
{
	MI_PIN = 0x00,
	MI_ONLINE = 0x50,
	MI_OFFLINE = 0x60,
	MI_FORCED_CVM = 0x80,
	MI_FORCED_PIN = 0x81,
	MI_FORCED_SIGNATURE = 0x82,
} FLX_MI;

enum Transaction_Type 
{
	TT_GOODS_SERVICE = 0x00,
	TT_CASH = 0x01,
	TT_GOODS_SERVICE_CASH = 0x09,
	TT_QUASI_CASH = 0x11,
	TT_REFUND = 0x20,
};

#ifndef WIN32
typedef unsigned long FLX_AMOUNT;	// Amount must be 4 bytes.
#else
typedef unsigned int FLX_AMOUNT;	// Use 32 bit Windows
#endif


/* BINARY RECEIPT INFO STRUCT RETURNED BY FLX_CALLBACK_GET_BINARY_RECEIPT */

#ifndef _TIME_T_DEFINED
#ifndef BUILDING_LINUX_DLL
typedef long time_t;        /* time value */
#endif
#define _TIME_T_DEFINED     /* avoid multiple def's of time_t */
#endif

/*
	The ReceiptStruct is data extracted from the receipt generated by the terminal.
	The Terminal must be configured to send the information to the dll.
	The struct is reset to all zeros by default, and only valid receipt information 
	is mapped into the struct. Consult Point for more information.
*/
/* enable structure packing */
#if defined(__GNUC__)
#define	PACKED	__attribute__((packed))
#else
#if defined(_WIN32)
#pragma pack(1)
#endif
#define	PACKED
#endif

//  see: NETS Electronic Receipt design document (JKP, august 2013)
//	Defined Scheme Ids
//	Value	ERCo
//	01	Kvittering.dk
//	02	eKvittering Aps
//	03  1F
//	20	Nets Denmark A/S
#define KVITTERINGDK_SCHEME		0x01
#define EKVITTERINGDK_SCHEME	0x02
#define NETSDK_SCHEME			0x20

#define NETS_HASH_MAX			3
#define	EHASH_LENGTH_MAX		32

// Placeholder for NETS ereceipt HASH values etc.
// See. OTRS Retrieve Hash Value command ('0019')
typedef struct NetsHashStruct_T
{
	unsigned char scheme_id;
	unsigned char algorithm_id;
	unsigned char salt_version;
	unsigned char len_hash;
	unsigned char hash[EHASH_LENGTH_MAX];
} PACKED NetsHash, *pNetsHash;

typedef struct NetsHashResStruct_T
{
	NetsHash	NetsEreceipts[NETS_HASH_MAX];	// NETS E-receipt NETS_HASH_MAX = 3				
} PACKED NetsHashRes, *pNetsHashRes;

typedef struct ReceiptStruct_T 
{
	time_t		gmtime;					// Windows (Unix) time stamp (gmt) (NOTE: 64 bit)
    FLX_AMOUNT	total;					// total amount in smallest currency unit
	FLX_AMOUNT	extra;					// extra -
	FLX_AMOUNT	fee;					// fee -
	FLX_AMOUNT	gratuity;				// gratuity -
    int		    currency;				// Currency code (208=DKK, 978=EUR, 840=USD  etc.)
	int		    Stan;					// PBS reference number converted from BCD code
    int		    PSAM_Creator;			// PSAM # 
	int		    PSAM_ID;				// PSAM ID #
	int		    MTI;					// PBS apacs transaction code
	int		    Asw1Asw2;				// PSAM retur code (see OTRS specification)
	int		    ActionCode;				// Transaction result code 
	int			refNr;					// Transaction reference number passed from user
	
	char		pbsResult;				// PBS Transaction result
	char	    CvmStatus;				// Cardholder verification method (eg. if signature required) see EMV specs.
	char	    CardDataSource;			// Magnetic- or chip card
	char	    TR;						// Transaction request
	char	    nota;					// True if signature on receipt
	char	    copy;					// True if receipt copy
    char	    mt;						// EMV decline code see EMV specification
	char	    psn;					// EMV pan sequence number see EMV specification
	char	    lenPAN;					// Length of PAN
    char	    lenAID;					// Length of AID

	char	    PAN[10];				// Primary account number BCD code
	char		AID[16];				// EMV application ident.
	char	    ATC[2];					// EMV value see EMV specification
	char	    AED[3];					// EMV value see EMV specification
	char	    ARC[2];					// EMV value see EMV specification
    char	    PosEntryMode[3];		// Transaktionen information see OTRS and EMV specification.

    int			number;					// Merchant PBS number
	char	    authorizationCode[6+1];	// PBS authorization code as string
	char	    CardName[16+1];			// Card name as string            (+1 indicate string termination '\0')
	char	    TermIdent[8+1];			// Terminal ID string
    char	    name[18+1];				// Merchant name as string
    char	    city[16+1];				// Merchant city as string
    char	    address[24+1];			// Merchant address as string
    char	    zip_code[8+1];			// Merchant zip code as string
    char	    phone[24+1];			// Merchant phone number as string
    char	    cvr[12+1];				// Merchant CVR number as string
    int		    DCCcurrency;			// DCC Currency code (208=DKK, 978=EUR, 840=USD  etc.)
	char		DCCrate[8+1];			// DCC rate in ascii (e.g. '0.138400')
	int			CardCRC;				// Card reconciliation number
	char		CancellationAllowed;	// Cancellation of this transaction allowed within PBS time limit
	char		BatchNumber[12+1];		// Batch Number as string
	FLX_AMOUNT	vat;
	char		Ereceipt[64+1];			// E-receipt point ekvittering.dk
	FLX_AMOUNT	saldo;
	FLX_AMOUNT	dcctotal;
	FLX_AMOUNT	dccfee;
	FLX_AMOUNT	dccgratuity;
	FLX_AMOUNT	cashback;
	char		expiredate[2];			// local card expire date as 2 byte (first: year (bcd) 0-99, second: month (bcd) 0-11)
	int			NetsEreceiptLen;
	NetsHashRes	NetsEreceipts;			// NETS E-receipt NETS_HASH_MAX = 3
	char		Kreceipt[64+1];			// e-receipt point kvittering.dk
    char        vasJsonReceipt[700];
	char        tcc[3];                                                       // Transaction Condition Code

} PACKED ReceiptStruct, *pReceiptStruct;

typedef struct PAN_T 
{
    unsigned char len;
    unsigned char data[10];
} PACKED PAN, *pPAN;

/* Structs used by the flxGetDCproperties callback method	*/
typedef struct STAN_T 
{
	unsigned char data[3];
} PACKED STAN, *pSTAN;

typedef struct DC_RESULT_T 
{
	STAN		refStan;			// unique psam transaction identifier
	FLX_AMOUNT	amount;
	int			currc;				// curreccy code
	int			curre;				// currency exponent
	char		DTHR[5];			// PBS timestamp year, month, day, hour, minute
} PACKED DC_RESULT, *pDC_RESULT;

// Additional Transaction Information (used in SEP/OPEL)
typedef struct DC_ATI_T 
{
	char	len;
	char	data[15];
} PACKED DC_ATI, *pDC_ATI;

typedef struct AdviceStruct_T 
{
	time_t		gmtime;					// Windows (Unix) time stamp (gmt) (NOTE: 64 bit)
    STAN		orgstan;				// total amount in smallest currency unit
    STAN		refstan;				// total amount in smallest currency unit
	int			advice;
	int			otrs;
	int			Card_Recon_Name;
	int			Card_Recon_ID;
} PACKED AdviceStruct, *pAdviceStruct;
typedef struct ExtraAppReplyStruct_T
{
	unsigned char app_no;
	unsigned char version;
	unsigned char error;
	unsigned char last_block;
	unsigned char data[1000];
} PACKED ExtraAppReplyStruct;	// Extra application data block

typedef struct
{
	unsigned char resp_mode;	// 0x00  Request - Append EIE to PSAM (PSAM start with empty EIE)
							// 0x01  Advice  - Clear EIE in PSAM before update with this
							// 0x02  Advice  - Append to EIE hold by PSAM
							// 0x03  Advice  - Clear EIE in PSAM only
	                        // 0x04 - Request - Clear EIE in PSAM only
							// 0x05  Response from Host to the Request EIE
    int	length_ie;			// Length of SW IE part of data
    int	length_eie;			// Length of EIE part of data
	unsigned char data[512];	// 0..length_ie+length_eie
} PACKED EIE_DATA_BLOCK_TYPE;	// Extended Issuer Envelope Data Block

typedef struct
{
	unsigned char data[1004];
} PACKED EXTRA_APP_DATA_BLOCK_TYPE;			// Extra application data block

typedef struct
{
	unsigned char data[1004];						// First byte is appication number
} PACKED EXTRA_APP_REPLY_DATA_BLOCK_TYPE;	// Extra application data block

#if defined(_WIN32)
#pragma pack()
#undef PACKED
#endif


/*
	These CALLBACK FUNCTIONS are defined and coded in the ECR software
	They are documented here for your convinience
	These functions must be implemented by the ECR when mandatory or when used
	(e.i. activated and defined using flxInitCallback).
*/
/*
	Function:	pcbPrintReceipt
				printReceipt prints a formatted receipt or report.

	Prototype:	int  printReceipt(unsigned int receipt_status, const char* cText);

	Input:		receipt_status	- 0x02 receipt requires verification
								  0x01 if there are missing segments (e.g. during ADMIN_REPORT_LOG) 
								  0x00 if it is the last/whole segment (all transaction receipts).
								       must always return RECEIPT_OK
								  (Clarification: use bit 0 (LSB) to cut between receipts)

				cText			- receipt text coded as ISO 8859-15 and 0 terminated.
								  maximum length in 1000 characters incl. '\0'. 
								  If a larger text must be printed divided in smaller segments.

	Output:						-	
	Return:		RECEIPT_OK		- receipt printed OK

*/
/*
	Function:	pcbAbort
				printAbort asks if operator whish to abort currect transaction.
				During transactions the flex-driver repeatedly calls the abortTransaction function. 
				Notice that even though the ECR tries to abort a Transaction, it might not be aborted. 
				It is the Terminal that decides if the Transaction is aborted, or not. 
				This function must be implement

	Prototype:	int  pcbAbort (void);

	Input:		-

	Output:		-					-	
	Return:		OPERATORWISHTOABORT			- Abort
				OPERATORDONOTWISHTOABORT	- do not abort


*/
/*
	Function:	pcbSetCarddata
				pcbSetCarddata gives cardnumber cardtype and crc (card reconcilliation counter id).
				Can be used by the ECR to reject certain types of credit cards, 
				e.g. if a customer wishes to get some cash together with the purchased item, 
				then the ECR should ensure that the swiped card isn't a credit card.

	Prototype:	int  setCarddata(const char *cPan, int cardtype);

	Input:		*cPan		- pointer to 8 first numbers in cardnumber(pan),crc in ascii
							  (e.g. 54130347,1 (1 is dankort))
				cardtype	- PBS_CARD		(card handled by PBS)
							  OTHER_CARD	(local card handled by ECR)

	Output:		-	
	Return:		CARDNUMBER_OK			- cardnumber ok	
				CARDNUMBER_OK_CONFIRM	- cardnumber (e.g. local card) OK
				CARDNUMBER_NOTOK		- cardnumber not ok


*/
/*
	Function:	pcbDisplayStatus
				pcbDisplayStatus gives an ascii status line

	Prototype:	void  pcbDisplayStatus(const char cStat[21], unsigned int line);

	Input:		cStat[]		- pointer 20 char ascii string plus '\0' termination coded as ISO 8859-15 
				line		- suggested status line number (1-4)

	Output:		-	
	Return:		-

*/
/*
	Function:	pcbVerSigConfirm
				pcbVerSigConfirm asks for signature approval on a signaturetransaction.
				E.g. it could be implemented so the operator e.g. presses 'F1' for approval and 'F2' for rejected. 
				The PSAM decides if this functionality is used or not. 
				If it is not called and the transaction is approved, both receipts will be sent from the terminal straight away. 
				If the signature afterwards seems fake, the operator must run a refund. 

	Prototype:	int  pcbVerSigConfirm (void);

	Input:		-
	Output:		-	
	Return:		SIGNATURE_OK
				SIGNATURE_NOTOK
				SIGNATURE_NOTKNOWNYET

*/
/*
	Function:	pcbAdviceFlag
				pcbAdviceFlag signals that balancing/endofday routine is nessesary
				Meaning that the ECR must not be able to run further transactions before the endofday routine is run and approved. 
				This function is mandatory to implement


	Prototype:	void  pcbAdviceFlag (void);

	Input:		-
	Output:		-	
	Return:		-

*/
/*
	Function:	pcbHostAdvice
				pcbHostAdvice signals that the aquire host has send a message
				This function is optional to implement and is an extention to pcbAdviceFlag


	Prototype:	void  pcbHostAdvice (unsigned char *tag, char *text);

	Input:		*tag pointer to host tag (0xC9, 0xCA...)
				*text pointer to host text string (20 chars max + delimiter '\0') 
	Output:		-	
	Return:		-

*/
/*
	Function:	pcbSetAmount
				pcbSetAmount sets the transaction amount
				If this function isn't initialised in the flxInitCallback function, 
				then the amount given in the flxCardTransaction is used.

	Prototype:	FLX_AMOUNT  pcbSetAmount(void);

	Input:		-
	Output:		-	
	Return:		FLX_AMOUNT - the transaction amount (always positive)
							 amount in lowest currency denomination (e.g. re). 

*/
/*
	Function:	pcbSetExtendedAmount
				pcbSetExtendedAmount sets an extended transaction amount (e.g. vat)

	Prototype:	FLX_AMOUNT  pcbSetAmount(FLX_AMOUNT_TYPE ext, CURR_CODES *curr);

	Input:		ext			- amount type 
	Output:		curr		- currency code	
	Return:		FLX_AMOUNT	- the extended transaction amount (always positive)
							  amount in lowest currency denomination (e.g. re). 

*/
/*
	Function:	pcbGetExtendedAmount
				pcbGetExtendedAmount gets an extended transaction amount (e.g. gratuity)

	Prototype:	void  pcbGetAmount(FLX_AMOUNT_TYPE ext, FLX_AMOUNT amount, CURR_CODES *curr);

	Input:		ext			- amount type
				amount		- amount
				curr		- currency code
	Output:		-	
	Return:	 

*/
/*
	Function:	pcbGetAmountFee
				pcbGetAmountFee gives the fee calculated in the terminal.

	Prototype:	void  pcbGetAmountFee(FLX_AMOUNT fee);

	Input:		fee	- fee
	Output:		-	
	Return:		- 

*/
/*
	Function:	pcbSetAmountFee
				pcbSetAmountFee gives the fee calculated in the terminal.

	Prototype:	FLX_AMOUNT  pcbSetAmountFee(void);

	Input:		-
	Output:		-	
	Return:		FLX_AMOUNT - the transaction fee (always positive) calculated by the ECR
							 amount in lowest currency denomination (e.g. re).
							 if used in with pcbGetAmountFee, it overrules the terminal fee.

*/
/*
	Function:	pcbSetAmountGratuity
				pcbSetAmountGratuity sets the transaction gratuity amount

	Prototype:	FLX_AMOUNT  pcbSetAmountGratuity(void);

	Input:		-
	Output:		-	
	Return:		FLX_AMOUNT - the transaction gratuity amount (always positive)
							 amount in lowest currency denomination (e.g. re). 

*/
/*
	Function:	pcbStatusTrace
				pcbStatusTrace 

	Prototype:	void  pcbStatusTrace (const char *trace);

	Input:		-
	Output:		-	
	Return:		-
*/
/*
	Function:	pcbStateTrace
				pcbStateTrace 

	Prototype:	void  pcbStateTrace (int state);

	Input:		-
	Output:		-	
	Return:		-
*/
/*
	Function:	pcbAdviceLog
				pcbAdviceLog sends copies of datastore operations to the ECR. 
				The transaction time is increased by approx half a second when using this function.

	Prototype:	void  adviceLog (unsigned int len, const unsigned char* data);

	Input:		len		- length of advice data
				*data	- pointer to advise data
	Output:		-	
	Return:		-
*/
/*
	Function:	pcbMenu
				pcbMenu is currently used to display information to the operator. 
				E.g. if or if not an EMV card is placed correctly in the terminal (command=0). 
				The function is also used for the operator to choose between EMV applications 
				in correlation with refund transactions (10 <= command >= 1).

	Prototype:	void  pcbMenu(unsigned char command, int timeout, unsigned char lines, const char menus[][21]);

	Input:		command		- command
							  1: Application List
							  2: EMV card inserted correct? (menu line 0 e.g. 'KORT ISAT KORREKT?'
							  3: DCC currency select (menu line 0 is <dccAmount>,<dccCurr>,<exp> '100000000,USD,2')
				timeout		- the ECR must answer before timeout value (using pcbMenuResult)
				lines		- number of lines in menu (max. 10)
				menus[][]	- menulines in ascii from 1 to 10
							  each line is 20 char plus terminating '\0'.
	Output:		-	
	Return:		-
*/
/*
	Function:	pcbMenuResult
				pcbMenuResult is called repetitively (like abort), to verify the choice made by the operator,
				when a menu is displayed on the ECR screen.

	Prototype:	int  pcbMenuResult(char *result);

	Input:		*result	- pointer to result holding selected line in menu
	Output:		- if EMV card inserted correct? use
				  *result = -1 (or 0) for no
				  *result = 1 for yes
				  if APPLICATION SELECT return
				  *result = selected menu line (1-10)

	Return:		0  - the answer isnt known yet (the operator hasnt pressed a key) and no timeout has occurred.
				1  - the chosen menu point must be entered in the result variable 
				     e.g. *result = 1 if first menu line is selected.
				     If a timeout has occurred, *result = -2 (used internally). 

*/
/*
	Function:	pcbGetReceipt
				pcbGetReceipt returns a 'binary' receipt struct defined in Receipttruct.

	Prototype:	void  pcbGetReceipt (ReceiptStruct *rs);

	Input:		*rs	- pointer to result struct holding binary receipt
	Output:		-	
	Return:		- 

*/
/*
	Function:	pcbEarlyStanPan
				get stan and pan as early as possible in transaction flow.
				This function is used with the flxGetDCProperties method.

	Prototype:	void  pcbEarlyStanPan(STAN* stan, PAN* pan, char *DTHR);

	Input:		*stan - pointer to hold stan
				*pan  - pointer to hold pan
				*DTHR - pointer to hold transaction timestamp in dthr format
	Output:		stan  - stan
				pan   - pan
	Return:		-
*/
/*
	Function:	pcbBreakIP
				break the ip idle wait

	Prototype:	int  pcbBreakIP(void);

	Input:		-
	Output:		-
	Return:		true if break
*/
/*
	Function:	pcbTimer		(not in linux .so version)
				break the ip idle wait

	Prototype:	int  pcbTimer(int millisec);

	Input:		millisec - timer value in milli seconds the timer was started with 
	Output:		- 
	Return:		-
*/
/*
	Function:	pcbPutToken
				Send a transaction token to terminal from ECR

	Prototype:	void  pcbPutToken(size_t *TokenLength, unsigned char *TokenData);

	Input:		*TokenLength -  Pointer to length of token
				*TokenData	 -	Token
	Output:		- 
	Return:		-
*/
/*
	Function:	pcbGetToken(size_t TokenLength, const unsigned char *TokenData, const char *TokenId)
				Send a transaction token from terminal to ECR
				Called from flxCardTransaction method

	Prototype:	int  pcbGetToken(size_t TokenLength, unsigned char *TokenData, const char *TokenId);

	Input:		TokenLength   -	length of token
				*Tokendata	  -	Token
				*TokenId	  - Unique token id
	Output:		- 
	Return:		RECEIPT_OK	  - Token OK
	            RECEIPT_NOTOK - Token not OK (something wrong with this token)
*/
/*
	Function:	pcbStopList
				Ask operator for transaction authorization
				Called from flxCardTransaction method

	Prototype:	int  pcbStopList(const char *Code);

	Input:		Code	- ptr to authorization code (issued by PBS)
	Output:		- 
	Return:		0		- the answer isnt known yet.
				1		- the pcbStopList Code is ready 
*/


/*  FLEX FUNCTIONS	*/

/*
	Function:	flxInitCallback
				initialize and activate callback function
	Input:		method_id	- id from FLX_CALLBACK enum
				method_ptr	- pointer to callback function
	Output:		-	
	Return:		FAILURE		- not OK
				SUCCESS		- OK.
*/
DllExport void flxInitCallback (
			FLX_CALLBACK method_id, 
			void* method_ptr);


/*
	Function:	flxConnect
				connect to terminal
				flxdrv.ini file used by function to determine connect type.
	Input:		dummy - internal use , set to 0
	Output:		-	
	Return:		- See CONNECT RETURN VALUES above
				FAILURE		- not OK
				SUCCESS		- OK.
*/
DllExport int flxConnect(
			int dummy);

/*
	Function:	flxDisconnect
				disconnect to terminal
				flxdrv.ini file used by function to determine disconnect type.
	Input:		-
	Output:		-	
	Return:		- See CONNECT RETURN VALUES above
				FAILURE		- not OK
				SUCCESS		- OK.
*/
DllExport int flxDisconnect(
			void);

/*
	Function:	flxOpen
				open session to terminal
				flxdrv.ini file used by function to determine open type.
	Input:		-
	Output:		-	
	Return:		- See CONNECT RETURN VALUES above
				OPEN_NO_RECEIPT - if open but terminal in no receipt state
				FAILURE			- not OK
				SUCCESS			- OK.
*/
DllExport int flxOpen(
			void);

/*
	Function:	flxClose
				close session to terminal
				flxdrv.ini file used by function to determine close type.
	Input:		-
	Output:		-	
	Return:		- See CONNECT RETURN VALUES above
				FAILURE		- not OK
				SUCCESS		- OK.
*/
DllExport int flxClose(
			void);


/*
	Function:	flxCardTransaction
				initiate a card transaction
	Input:		transactionType		- from enum FLX_TRANSTYPE
				amount				- transaction amount in smallest currency unit (eg. re)
				currencyCode		- currency code from CURR_CODES
				merchantInitiative	- from enum FLX_MI
									  see PBS OTRS 2.1 (table 9.3.37) documentation for more information
				refno				- your transaction reference number
									  (returned in ReceiptStruct)
	Output:		*error				- ptr to error code
									  0x00 (SUCCESS)
									  ERR_UNKNOWN
									  ERR_NO_CONNECTION
									  ERR_NO_RECEIPT
	Return:		FAILURE				- not OK
				SUCCESS				- OK.
				LOCALCARD_NOT_OK	- Local Card transaction not approved (use flxCompleteExtCardTransaction to complete)
				LOCALCARD_OK		- Local Card transaction approved (use flxCompleteExtCardTransaction to complete)
*/
DllExport int flxCardTransaction(
			FLX_TRANSTYPE transactionType,
			FLX_AMOUNT amount,
			int currencyCode,
			unsigned char merchantInitiative,
			const unsigned int refno,
			unsigned int *error);

/*
	Function:	flxCompleteExtCardTransaction
				used to complete a extern (local) card transaction
	Input:		result				- 0x00 approves the transaction
									- 0x01 rejects the transaction
	Output:		error				- ERR_E codes	
	Return:		FAILURE				- PBS transaction not approved
				SUCCESS				- PBS transaction approved
*/
DllExport int flxCompleteExtCardTransaction(
			int result);

/*
	Function:	flxAdministration
				used to initiate administrative functions in the terminal
	Input:		func		-	function from FLX_ADMIN_FUNCTION enum

	Output:		*error		-	ptr to error code
								0x00 (SUCCESS)
								ERR_UNKNOWN
								ERR_NO_CONNECTION
								ERR_NO_RECEIPT
	Return:		FAILURE		- not OK
				SUCCESS		- OK.
*/
DllExport int flxAdministration(
			FLX_ADMIN_FUNCTION func, 
			unsigned int *error);

/*
	Function:	flxGetDllVersion
				gets dll windows product version
	Input:		-
	Output:		error	- ERR_E codes	
	Return:		0x0000	- no version
				0xMMmm	- MM Major Version truncated to 2x16 bit
						  mm Minor version truncated to 2x16 bit
*/
DllExport int flxGetDllVersion(
			void);

/*
	Function:	flxGetTerminalState
				gets the state of the terminal
	Input:		-
	Output:		-	
	Return:		terminal state, consult flxdrv.dll manual
*/
DllExport int flxGetTerminalState(
			void);


/*
	Function:	flxSetConfiguration
				sets the driver configuration parameter
	Input:		func	-	selects parameter
	Output:		-	
	Return:		FAILURE		- not OK
				SUCCESS		- OK.
*/
DllExport int flxSetConfiguration(
			int func, ...);

/*
	Function:	flxSetTrace
				set dll trace level
	Input:		traceLevel - see trace level defines
	Output:		-	
	Return:		-
*/
DllExport void flxSetTrace(
			int traceLevel);

/*
	Function:	flxGetTrace
				return dll trace level
	Input:		-
	Output:		-	
	Return:		tracelevel - see trace level defines
*/
DllExport int flxGetTrace(
			void);

/*
	Function:	int flxGetDCProperties(char stanPan, STAN_PAN *stanPan)
				search the PSAM (version >= 50) for status of previous transactions. 
	Input:		cmd     - if 0x80 use stan as search key
				pStan	- pointer to stan search key

	Output:		dcRes   - search result as DC_RESULT struct if return SUCCESS
						  memset to 0 if return is FAILURE

	Return:		!= 0 - if the PSAM cannot be accessed at this time or no search match (psam code)
				0	 - OK.

*/
DllExport int flxGetDCProperties(
			char cmd, 
			STAN *pStan, 
			DC_RESULT *dcRes);


/*
	Function:	int flxSetDCProperties(char stanPan, STAN_PAN *stanPan)
				search the PSAM (version >= 50) for status of previous transactions. 
	Input:		
	
	Return:		!= 0 - if the PSAM cannot be accessed at this time or no search match (psam code)
				0	 - OK.

*/

DllExport int flxSetDCProperties(
			char command, 
			DC_ATI *dcati);

/*
	Function:	int flxGetID(void)
				Return unique Point terminal ID. 
	Input:		
	
	Return:		!= 0 - Unique Point terminal ID  (e.g. 990125)
				0	 - No ID not found.

*/

DllExport int flxGetID(void);

/*
	Function:	int flxIdleIProuting
				Continue idle 'IP routing' (forwarding of ip to/from serial port).

	Input:		syncronize - if 0 the function continues untill pbcBreakIP returns 1
	                         if 1 the function looks for a syncframe received from terminal
							      and continues untill pcbBreakIP callback return 1
							  if 2 the function looks for card swiped (inserted) and return

	Output:		error   -  error code (e.g. not connected)

	Return:		FAILURE - not OK
				SUCCESS	- OK.
				CARD_SWIPED

*/
DllExport int flxIdleIPforwarding(
			int syncronize, unsigned int *error);

/*
	Function:	int flxExtraReply(struct ExtraAppReplyStruct_T *ExtraReplyData, int ExtraReplyDataLength)
				
	Input:      struct ExtraAppReplyStruct_T
					app_no		1 byte
					version		1 byte
					error		1 byte
					last_block	1 byte
					data		0-1000 bytes
				ExtraReplyDataLength	legal values 4-1004

*/

DllExport int flxExtraReply(struct ExtraAppReplyStruct_T *ExtraAppData, int extra_length);

/*
	Function:	int flxGetSetEcrExtendedFunctions(unsigned char command, int *resultvalue)
				
	Input:      command
				ADMIN_GET_ECR_EXTENDED_FUNCTIONS = 0x85
				ADMIN_SET_ECR_EXTENDED_FUNCTIONS = 0x84

				resultvalue	- ERR_IP_ROUTING if no connection.
				If ADMIN_SET_ECR_EXTENDED_FUNCTIONS
				  Pointer to variable containing value to set EcrExtendedFunctions to in terminal
			    If ADMIN_GET_ECR_EXTENDED_FUNCTIONS
				  Pointer to variable to return value of terminals EcrExtendedFunction


	Return:		!= 0 - Unable to SET/GET value of terminals EcrExtendedFunctions
				0	 - OK.

*/

DllExport int flxGetSetEcrExtendedFunctions(
			unsigned char command, 
			int *resultvalue);


/*
	Function:	char *flxReadToken(int what, char *tokenfile, int *error)
				Read a token and return its what data.

	Input:		tokenfile     - token file name

	Output:		*error
				ERR_TOKEN - not OK
				SUCCESS	- OK.
	
	Return:		what
				1: data : <type>,<ver>,<pan|aid>,<amount>,<curr>,<exp>,<VPKca>,<VK>,<ALGH>,<Merchant>,
				2: data : <feepct>,<cvm>,<OriginalRef>,
				3: dcc  : <selected><me_amount>,<me_amountfee>,<me_amountgratuity>,<dcccurr>,<exp>,<dccrate>,<ch_amount><ch_amountfee><ch_amountgratuity><clerkid>
				date is a buffer of at least 128 bytes

*/
DllExport char *flxReadToken(
		    int what,
			char *tokenfile, 
			unsigned int *error);


/*
	Function:	char *flxTerminalProperties(int command, char *str, size_t length, uint *error);
				get terminal properties
				the function must be called just after flxOpen() to return valid data
				else pTPROPERTIES are all zero

	Input:		command - 1: str is output of terminal settings
						  2: str is input with amount in ascii for Prereceipt ( 
						  3: str is input for datastore cleaning
						  4: set properties  (not implemented)
						  5: set is input to advice reporting (pbs)
						  6: batch
						  7: str is labels to be used with 1 terminal settings

				str		- pointer to csv ascii string
						  1:
						  2: <AMOUNT>,<CURRENCYCODE>
						  3: <PSAMID>,<RECORDID>,<FILENUMBER>,<STAN>
						  4:
						  5: <ADVICEFILENUMBER>

				length	- length of string str

	Output:		*error
				SUCCESS	- OK.
				FAILURE - not OK
	
	Return:		null if command not valid or str size error

				1: pointer to ascii csv str including properties
				   <TERMINALID>,<RECEIPTTYPE><GRATUITYMODEL>,<DCC>,<TOKEN>,<PREPAID>,KEYENTRY>,<OFFLINE>,<IPROUTING>,
				   <MERCHANTNUMBER>,MERCHANTNAME>,<MERCHANTCITY>,<MERCHANTADDRESS>,<MERCHANTZIP>,
				   <MERCHANTPHONE>,<MERCHANTBRN>
			    2: str
				3: str
				4:
				5: str
				(consult Point for more information)
							
*/

typedef enum pcommand_t
{
	ADMIN_PROPS_NONE,
	ADMIN_PROPS_GET,
	ADMIN_PROPS_PREPECEIPT,
	ADMIN_PROPS_DATASTORE,
	ADMIN_PROPS_SET,
	ADMIN_PROPS_ADVICERECON,
	ADMIN_PROPS_BATCH,
	ADMIN_PROPS_GET_LABELS,
	ADMIN_PROPS_EVENTLOG_SEND,
	ADMIN_PROPS_EVENTLOG_DELETE,
	ADMIN_PROPS_SETIP_SETTINGS,
	ADMIN_PROPS_GET_EXT_LABELS,
	ADMIN_PROPS_SETTELEDONE_SETTINGS,
	ADMIN_PROPS_USER_INPUT,

	ADMIN_PROPS_PRINT_RECEIPT_FROM_STAN=0xf0,
} FLX_PROPSCOMMAND;

DllExport char *flxTerminalProperties(FLX_PROPSCOMMAND command, char *str, size_t length, unsigned int *error);

/***
	Function:	int flxDropIp(char *ip_addr, char *terminal_ident, int to_do);
				Tell terminal to drop it's IP connection, making it ready for a new connect

	Input:		ip_addr				Terminals IP address
					ex. 10.0.0.156
				terminal_ident		Terminal identifier
					ex. 00990768	Must be 8 digits
				to_do				Action terminal has to perform
				    0:	DropIp		Dropping terminals IP connection
					1:  Reset		Restarts terminal

     Output:	1 - SUCCESS - OK.
				0 - FAILURE - not OK
***/
DllExport int flxDropIp(char *ip_addr, char *terminal_ident, int to_do);

/***
	The Linux lib use this function to determin if the timeouts used should be in seconds or milliseconds
	Default is milleseconds.
***/
int get_use_gettimeofday();

#ifdef __cplusplus
}
#endif

#endif // __FLEX_H
